@@ -36,7 +36,7 @@ import networkx as nx
 import networkx_query as nxq
 
 from .abc_property_graph import ABCPropertyGraph, PropertyGraphImportException, \
-    PropertyGraphQueryException, ABCGraphImporter
+    PropertyGraphQueryException, ABCGraphImporter, GraphFormat
 from .networkx_mixin import NetworkXMixin
 
 
@@ -321,6 +321,22 @@ class NetworkXPropertyGraph(ABCPropertyGraph, NetworkXMixin):
             ret.append(my_graph.nodes[n][ABCPropertyGraph.NODE_ID])
         return ret
 
+    def get_all_nodes_by_class_and_type(self, *, label: str, ntype: str) -> List[str]:
+        assert label is not None
+        assert ntype is not None
+        my_graph = self.storage.get_graph(self.graph_id)
+        graph_nodes = list(nxq.search_nodes(my_graph,
+                                            {'and': [
+                                                {'eq': [ABCPropertyGraph.GRAPH_ID, self.graph_id]},
+                                                {'eq': [ABCPropertyGraph.PROP_CLASS, label]},
+                                                {'eq': [ABCPropertyGraph.PROP_TYPE, ntype]}
+                                            ]
+                                            }))
+        ret = list()
+        for n in graph_nodes:
+            ret.append(my_graph.nodes[n][ABCPropertyGraph.NODE_ID])
+        return ret
+
     def list_all_node_ids(self) -> List[str]:
         """
         List all NodeID properties of nodes in a graph
@@ -342,7 +358,7 @@ class NetworkXPropertyGraph(ABCPropertyGraph, NetworkXMixin):
         self.storage.add_graph(new_graph_id, new_graph)
         return NetworkXPropertyGraph(graph_id=new_graph_id, importer=self.importer, logger=self.log)
 
-    def serialize_graph(self) -> str:
+    def serialize_graph(self, format: GraphFormat = GraphFormat.GRAPHML) -> str:
         """
         Serialize a given graph into GraphML string or return None
         if graph is not found
@@ -351,7 +367,17 @@ class NetworkXPropertyGraph(ABCPropertyGraph, NetworkXMixin):
         graph = self.storage.extract_graph(self.graph_id)
         graph_string = None
         if graph is not None:
-            graph_string = '\n'.join(nx.generate_graphml(graph))
+            if format == GraphFormat.GRAPHML:
+                graph_string = '\n'.join(nx.generate_graphml(graph))
+            elif format == GraphFormat.JSON_NODELINK:
+                json_object = nx.readwrite.node_link_data(graph)
+                graph_string = json.dumps(json_object)
+            elif format == GraphFormat.CYTOSCAPE:
+                json_object = nx.readwrite.cytoscape_data(graph)
+                graph_string = json.dumps(json_object)
+            else:
+                raise PropertyGraphQueryException(graph_id=self.graph_id, node_id=None,
+                                                  msg=f"Unsupported export graph format {format.name}")
         return graph_string
 
     def graph_exists(self) -> bool:
@@ -462,6 +488,9 @@ class NetworkXPropertyGraph(ABCPropertyGraph, NetworkXMixin):
             # filter second neighbors by label
             second_neighbors = self._filter_nodes_by_label(graph, second_neighbors, node2_label)
             if len(second_neighbors) > 0:
+                # remove self in case they are there
+                if real_node in second_neighbors:
+                    second_neighbors.remove(real_node)
                 second_neighbors_dict[n] = second_neighbors
 
         # convert to a list of two-member lists, converting internal IDs to guids
@@ -607,6 +636,34 @@ class NetworkXPropertyGraph(ABCPropertyGraph, NetworkXMixin):
                     new_props[k] = node_props[k]
         self.storage.get_graph(self.graph_id).nodes[real_node].update(new_props)
 
+    def get_stitch_nodes(self) -> List[str]:
+        my_graph = self.storage.get_graph(self.graph_id)
+        graph_nodes = list(nxq.search_nodes(my_graph,
+                                            {'and': [
+                                                {'eq': [ABCPropertyGraph.GRAPH_ID, self.graph_id]},
+                                                {'eq': [ABCPropertyGraph.PROP_STITCH_NODE, 'true']}
+                                            ]
+                                            }))
+        ret = list()
+        for n in graph_nodes:
+            ret.append(my_graph.nodes[n][ABCPropertyGraph.NODE_ID])
+        return ret
+
+    def check_node_unique(self, *, label: str, name: str):
+        """
+        Check no other node of this class/label and name exists
+        :param label:
+        :param name:
+        :return:
+        """
+        graph_nodes = list(nxq.search_nodes(self.storage.get_graph(self.graph_id),
+                                            {'and': [
+                                                {'eq': [ABCPropertyGraph.GRAPH_ID, self.graph_id]},
+                                                {'eq': [ABCPropertyGraph.PROP_NAME, name]},
+                                                {'eq': [ABCPropertyGraph.PROP_CLASS, label]}
+                                            ]}))
+        return len(graph_nodes) == 0
+
 
 class NetworkXGraphStorage:
     """
@@ -630,10 +687,8 @@ class NetworkXGraphStorage:
             # check this graph_id isn't already present
             existing_graph_nodes = list(nxq.search_nodes(self.graphs, {'eq': [ABCPropertyGraph.GRAPH_ID, graph_id]}))
             if len(existing_graph_nodes) > 0:
-                # graph already present, warn and exit
-                if self.log is not None:
-                    self.log.warn('Attempting to insert a graph with the same GraphID, skipping')
-                return
+                # graph already present, delete it so we can replace
+                self.del_graph(graph_id)
             # relabel incoming graph nodes to integers, then merge
             temp_graph = nx.convert_node_labels_to_integers(graph, first_label=self.start_id)
             # set/overwrite GraphID property on all nodes
@@ -650,10 +705,8 @@ class NetworkXGraphStorage:
             # check this graph_id isn't already present
             existing_graph_nodes = list(nxq.search_nodes(self.graphs, {'eq': [ABCPropertyGraph.GRAPH_ID, graph_id]}))
             if len(existing_graph_nodes) > 0:
-                # graph already present, warn and exit
-                if self.log is not None:
-                    self.log.warn('Attempting to insert a graph with the same GraphID, skipping')
-                return
+                # graph already present, delete so we can replace
+                self.del_graph(graph_id)
             # relabel incoming graph nodes to integers, then merge
             temp_graph = nx.convert_node_labels_to_integers(graph, first_label=self.start_id)
             self.start_id = self.start_id + len(temp_graph.nodes())
