@@ -13,6 +13,10 @@ import fim.slivers.interface_info as iinfo
 import fim.slivers.capacities_labels as caplab
 import fim.slivers.delegations as dlg
 import fim.graph.abc_property_graph as abcpg
+import fim.slivers.path_info as pinfo
+import fim.slivers.gateway as gw
+import fim.slivers.tags as tgs
+import fim.slivers.measurement_data as mdata
 
 # push some definitions up to simplify import management
 
@@ -44,3 +48,9 @@ DelegationType = dlg.DelegationType
 Pool = dlg.Pool
 Pools = dlg.Pools
 GraphFormat = abcpg.GraphFormat
+Gateway = gw.Gateway
+PathInfo = pinfo.PathInfo
+ERO = pinfo.ERO
+Tags = tgs.Tags
+MeasurementData = mdata.MeasurementData
+FreeCapacity = caplab.FreeCapacity
