@@ -28,6 +28,7 @@ from typing import Dict, Any, List, Tuple
 
 from fim.view_only_dict import ViewOnlyDict
 from .node import Node
+from .model_element import TopologyException
 from .interface import Interface
 
 from ..graph.abc_property_graph import ABCPropertyGraph
@@ -61,7 +62,7 @@ class CompositeNode(Node):
         existing_node_id = self.topo.graph_model.find_node_by_name(node_name=name,
                                                                    label=str(ABCPropertyGraph.CLASS_CompositeNode))
         if existing_node_id != node_id:
-            raise RuntimeError(f'Composite Node name {name} is not unique within the topology')
+            raise TopologyException(f'Composite Node name {name} is not unique within the topology')
 
     def __list_components(self) -> ViewOnlyDict:
         """
@@ -96,6 +97,8 @@ class CompositeNode(Node):
         :return:
         """
         # immediately-attached interfaces only for composite nodes
+        # FIXME: note that due to interface name collisions this may not return
+        # all the interfaces. Better to use list_of_interfaces
         node_if_list = self.topo.graph_model.get_all_node_or_component_connection_points(parent_node_id=self.node_id)
         direct_interfaces = dict()
         for nid in node_if_list:
@@ -103,12 +106,17 @@ class CompositeNode(Node):
             direct_interfaces[i.name] = i
         return ViewOnlyDict(direct_interfaces)
 
-    def __list_of_interfaces(self) -> Tuple[Any]:
+    def __list_of_interfaces(self) -> Tuple[Interface]:
         """
         List all interfaces of node and its components
         :return:
         """
-        return tuple(self.__list_interfaces().values())
+        node_if_list = self.topo.graph_model.get_all_node_or_component_connection_points(parent_node_id=self.node_id)
+        direct_interfaces = list()
+        for nid in node_if_list:
+            i = self.__get_interface_by_id(nid)
+            direct_interfaces.append(i)
+        return tuple(direct_interfaces)
 
     @property
     def components(self):
