@@ -29,16 +29,18 @@ from typing import Dict, Any, List, Tuple
 import uuid
 
 from fim.view_only_dict import ViewOnlyDict
-from .model_element import ModelElement, ElementType
+from .model_element import ModelElement, ElementType, TopologyException
 from .component import Component, ComponentType
-from .switch_fabric import SwitchFabric
+from .network_service import NetworkService
 from .interface import Interface
 
 from ..graph.abc_property_graph import ABCPropertyGraph, PropertyGraphQueryException
 
 from ..slivers.network_node import NodeSliver
 from ..slivers.network_node import NodeType
-from ..slivers.switch_fabric import SFLayer
+from ..slivers.network_service import ServiceType, NetworkServiceInfo
+from ..slivers.component_catalog import ComponentModelType
+from ..slivers.capacities_labels import CapacityHints, Location, ReservationInfo
 
 
 class Node(ModelElement):
@@ -47,16 +49,16 @@ class Node(ModelElement):
     return various dictionaries or lists:
     node.components - returns a dictionary of components by name
     node.interfaces - returns a dictionary of interfaces by name
-    interface_list - returns a list of interfaces
-    direct_interfaces - returns a dictionary of direct interfaces, i.e. those
+    node.interface_list - returns a list of interfaces
+    node.direct_interfaces - returns a dictionary of direct interfaces, i.e. those
     not attached to components (mostly relevant to switches, not servers
     or VMs)
-    switch_fabrics - returns a dictionary of switch fabrics (mostly relevant
+    network_services - returns a dictionary of network_services (mostly relevant
     to switches, not servers or VMs)
     """
 
     def __init__(self, *, name: str, node_id: str = None, topo: Any, etype: ElementType = ElementType.EXISTING,
-                 ntype: NodeType = None, site: str = None, **kwargs):
+                 ntype: NodeType = None, site: str = None, ns_info: NetworkServiceInfo = None, **kwargs):
         """
         Don't call this method yourself, call topology.add_node()
         node_id will be generated if not provided for experiment topologies
@@ -67,6 +69,7 @@ class Node(ModelElement):
         :param etype: is this supposed to exist or new should be created
         :param ntype: node type if it is new
         :param site: node site
+        :param ns_info: NetworkServiceInfo structure
         :param kwargs: any additional properties
         """
         assert name is not None
@@ -77,22 +80,22 @@ class Node(ModelElement):
             # node id myst be specified for new nodes in substrate topologies
             if str(topo.__class__) == "<class 'fim.user.topology.SubstrateTopology'>" and \
                     node_id is None:
-                raise RuntimeError("When adding new nodes to substrate topology nodes you must specify static Node ID")
+                raise TopologyException("When adding new nodes to substrate topology nodes you must specify static Node ID")
             if node_id is None:
                 node_id = str(uuid.uuid4())
             super().__init__(name=name, node_id=node_id, topo=topo)
             if ntype is None:
-                raise RuntimeError("When creating nodes you must specify NodeType")
+                raise TopologyException("When creating nodes you must specify NodeType")
             if site is None:
-                raise RuntimeError("When creating nodes you must specify site")
+                raise TopologyException("When creating nodes you must specify site")
 
             sliver = NodeSliver()
             sliver.node_id = self.node_id
             sliver.set_name(self.name)
             sliver.set_type(ntype)
             sliver.set_site(site)
+            sliver.network_service_info = ns_info
             sliver.set_properties(**kwargs)
-
             self.topo.graph_model.add_network_node_sliver(sliver=sliver)
         else:
             assert node_id is not None
@@ -101,7 +104,108 @@ class Node(ModelElement):
             existing_node_id = self.topo.graph_model.find_node_by_name(node_name=name,
                                                                        label=str(ABCPropertyGraph.CLASS_NetworkNode))
             if existing_node_id != node_id:
-                raise RuntimeError(f'Node name {name} is not unique within the topology')
+                raise TopologyException(f'Node name {name} is not unique within the topology')
+
+    @property
+    def site(self):
+        return self.get_property('site') if self.__dict__.get('topo', None) is not None else None
+
+    @site.setter
+    def site(self, value: str):
+        if self.__dict__.get('topo', None) is not None:
+            self.set_property('site', value)
+
+    @property
+    def type(self):
+        return self.get_property('type') if self.__dict__.get('topo', None) is not None else None
+
+    @property
+    def image_type(self):
+        return self.get_property('image_type') if self.__dict__.get('topo', None) is not None else None
+
+    @image_type.setter
+    def image_type(self, value: str):
+        if self.__dict__.get('topo', None) is not None:
+            self.set_property('image_type', value)
+
+    @property
+    def image_ref(self):
+        return self.get_property('image_ref') if self.__dict__.get('topo', None) is not None else None
+
+    @image_ref.setter
+    def image_ref(self, value: str):
+        if self.__dict__.get('topo', None) is not None:
+            imtype = self.get_property('image_type')
+            # image ref and type have fate-sharing - neither can be null to be written into a graph
+            self.set_properties(image_ref=value, image_type=imtype)
+
+    @property
+    def management_ip(self):
+        # no user-facing setter for this one
+        return self.get_property('management_ip') if self.__dict__.get('topo', None) is not None else None
+
+    @property
+    def capacity_hints(self):
+        return self.get_property('capacity_hints') if self.__dict__.get('topo', None) is not None else None
+
+    @capacity_hints.setter
+    def capacity_hints(self, value: CapacityHints):
+        if self.__dict__.get('topo', None) is not None:
+            self.set_property('capacity_hints', value)
+
+    @property
+    def reservation_info(self):
+        return self.get_property('reservation_info') if self.__dict__.get('topo', None) is not None else None
+
+    @reservation_info.setter
+    def reservation_info(self, value: ReservationInfo):
+        if self.__dict__.get('topo', None) is not None:
+            self.set_property('reservation_info', value)
+
+    @property
+    def location(self):
+        return self.get_property('location') if self.__dict__.get('topo', None) is not None else None
+
+    @location.setter
+    def location(self, value: Location):
+        if self.__dict__.get('topo', None) is not None:
+            self.set_property('location', value)
+
+    @property
+    def components(self):
+        return self.__list_components()
+
+    @property
+    def interfaces(self):
+        return self.__list_interfaces()
+
+    @property
+    def interface_list(self):
+        return self.__list_of_interfaces()
+
+    @property
+    def direct_interfaces(self):
+        return self.__list_direct_interfaces()
+
+    @property
+    def network_services(self):
+        return self.__list_network_services()
+
+    @property
+    def boot_script(self):
+        return self.get_property('boot_script') if self.__dict__.get('topo', None) is not None else None
+
+    @boot_script.setter
+    def boot_script(self, value: str):
+        if self.__dict__.get('topo', None) is not None:
+            self.set_property('boot_script', value)
+
+    def get_sliver(self) -> NodeSliver:
+        """
+        Get a deep sliver representation of this node from graph
+        :return:
+        """
+        return self.topo.graph_model.build_deep_node_sliver(node_id=self.node_id)
 
     def get_property(self, pname: str) -> Any:
         """
@@ -116,11 +220,14 @@ class Node(ModelElement):
 
     def set_property(self, pname: str, pval: Any):
         """
-        Set a node property
+        Set a node property or unset if pval is None
         :param pname:
         :param pval:
         :return:
         """
+        if pval is None:
+            self.unset_property(pname)
+            return
         node_sliver = NodeSliver()
         node_sliver.set_property(prop_name=pname, prop_val=pval)
         # write into the graph
@@ -143,70 +250,75 @@ class Node(ModelElement):
     def list_properties() -> Tuple[str]:
         return tuple(NodeSliver.list_properties())
 
-    def add_component(self, *, name: str, node_id: str = None, ctype: ComponentType,
-                      model: str,  switch_fabric_node_id: str = None, interface_node_ids=None,
-                      **kwargs) -> Component:
+    def add_component(self, *, name: str, node_id: str = None, ctype: ComponentType = None,
+                      model: str = None, model_type: ComponentModelType = None,
+                      network_service_node_id: str=None, interface_node_ids=None,
+                      interface_labels=None, **kwargs) -> Component:
         """
         Add a component of specified type, model and name to this node. When working with substrate
-        topologies you must specify the switch_fabric_node_id and provide a list of interface node ids.
+        topologies you must specify the network_service_node_id and provide a list of interface node ids.
         :param name:
         :param node_id:
-        :param ctype:
-        :param model:
-        :param name:
-        :param switch_fabric_node_id:
-        :param interface_node_ids:
+        :param ctype: ComponentType
+        :param model: Model string (exact match)
+        :param  model_type: ComponentModelType (combines ComponentType and Model)
+        :param network_service_node_id: network service identifier for substrate models
+        :param interface_node_ids: interface identifiers for substrate models
+        :param interface_labels: list of labels for interfaces in substrate models
         :param kwargs: additional properties of the component
         :return:
         """
         assert name is not None
         # make sure name is unique within the node
         if name in self.__list_components().keys():
-            raise RuntimeError('Component names must be unique within node.')
+            raise TopologyException('Component names must be unique within node.')
         # add component node and populate properties
         c = Component(name=name, node_id=node_id, topo=self.topo, etype=ElementType.NEW,
-                      ctype=ctype, model=model, switch_fabric_node_id=switch_fabric_node_id,
-                      interface_node_ids=interface_node_ids, parent_node_id=self.node_id, **kwargs)
+                      ctype=ctype, model=model, comp_model=model_type,
+                      network_service_node_id=network_service_node_id,
+                      interface_node_ids=interface_node_ids, interface_labels=interface_labels,
+                      parent_node_id=self.node_id, **kwargs)
         return c
 
-    def add_switch_fabric(self, *, name: str, node_id: str = None, layer: SFLayer):
+    def add_network_service(self, *, name: str, node_id: str = None, nstype: ServiceType, **kwargs) -> NetworkService:
         """
-        Add a switch fabric to node (mostly needed in substrate topologies)
+        Add a network service to node ( needed in substrate topologies)
         :param name:
         :param node_id:
         :param layer:
+        :param kwargs:
         :return:
         """
         assert name is not None
         # make sure name is unique within the node
-        if name in self.__list_switch_fabrics().keys():
-            raise RuntimeError('SwitchFabric names must be unique within node.')
-        sf = SwitchFabric(name=name, node_id=node_id, layer=layer, parent_node_id=self.node_id,
-                          etype=ElementType.NEW, topo=self.topo)
-        return sf
+        if name in self.__list_network_services().keys():
+            raise TopologyException('NetworkService names must be unique within node.')
+        ns = NetworkService(name=name, node_id=node_id, parent_node_id=self.node_id, topo=self.topo,
+                            etype=ElementType.NEW, nstype=nstype, **kwargs)
+        return ns
 
     def remove_component(self, name: str) -> None:
         """
-        Remove a component from the node (and switch fabrics and their interfaces)
+        Remove a component from the node (and network services and their interfaces)
         :param name:
         :return:
         """
         assert name is not None
         node_id = self.topo.graph_model.find_component_by_name(parent_node_id=self.node_id,
                                                                component_name=name)
-        self.topo.graph_model.remove_component_with_sfs_cps_and_links(node_id=node_id)
+        self.topo.graph_model.remove_component_with_nss_cps_and_links(node_id=node_id)
 
-    def remove_switch_fabric(self, name: str) -> None:
+    def remove_network_service(self, name: str) -> None:
         """
-        Remove a switch fabric from the node (and all its interfaces)
+        Remove a network service from the node (and all its interfaces)
         :param name:
         :return:
         """
         assert name is not None
 
-        node_id = self.topo.graph_model.find_sf_by_name(parent_node_id=self.node_id,
-                                                        sfname=name)
-        self.topo.graph_model.remove_sf_with_cps_and_links(node_id=node_id)
+        node_id = self.topo.graph_model.find_ns_by_name(parent_node_id=self.node_id,
+                                                        nsname=name)
+        self.topo.graph_model.remove_ns_with_cps_and_links(node_id=node_id)
 
     def __get_component_by_name(self, name: str) -> Component:
         """
@@ -231,27 +343,27 @@ class Node(ModelElement):
         return Component(name=node_props[ABCPropertyGraph.PROP_NAME], node_id=node_id,
                          topo=self.topo)
 
-    def __get_sf_by_name(self, name: str) -> SwitchFabric:
+    def __get_ns_by_name(self, name: str) -> NetworkService:
         """
-        Find SwitchFabric of a node by its name, return SwitchFabric object
+        Find NetworkService of a node by its name, return NetworkService object
         :param name:
         :return:
         """
         assert name is not None
-        node_id = self.topo.graph_model.find_sf_by_name(parent_node_id=self.node_id, component_name=name)
-        return SwitchFabric(name=name, node_id=node_id, topo=self.topo)
+        node_id = self.topo.graph_model.find_ns_by_name(parent_node_id=self.node_id, component_name=name)
+        return NetworkService(name=name, node_id=node_id, topo=self.topo)
 
-    def __get_sf_by_id(self, node_id: str) -> SwitchFabric:
+    def __get_ns_by_id(self, node_id: str) -> NetworkService:
         """
-        Get a switch fabric of a node by its node_id, return SwitchFabric object
+        Get a network service of a node by its node_id, return NetworkService object
         :param node_id:
         :return:
         """
         assert node_id is not None
         _, node_props = self.topo.graph_model.get_node_properties(node_id=node_id)
         assert node_props.get(ABCPropertyGraph.PROP_NAME, None) is not None
-        return SwitchFabric(name=node_props[ABCPropertyGraph.PROP_NAME], node_id=node_id,
-                            topo=self.topo)
+        return NetworkService(name=node_props[ABCPropertyGraph.PROP_NAME], node_id=node_id,
+                              topo=self.topo)
 
     def __get_interface_by_id(self, node_id: str) -> Interface:
         """
@@ -280,17 +392,17 @@ class Node(ModelElement):
             ret[c.name] = c
         return ViewOnlyDict(ret)
 
-    def __list_switch_fabrics(self) -> ViewOnlyDict:
+    def __list_network_services(self) -> ViewOnlyDict:
         """
-        List all switch fabric children of a node as a dictionary organized
-        by switch fabric name. Modifying the dictionary will not affect
-        the underlying model, but modifying Components in the dictionary will.
+        List all network service children of a node as a dictionary organized
+        by network service name. Modifying the dictionary will not affect
+        the underlying model, but modifying NetworkServices in the dictionary will.
         :return:
         """
-        node_id_list = self.topo.graph_model.get_all_network_node_or_component_sfs(parent_node_id=self.node_id)
+        node_id_list = self.topo.graph_model.get_all_network_node_or_component_nss(parent_node_id=self.node_id)
         ret = dict()
         for nid in node_id_list:
-            c = self.__get_sf_by_id(nid)
+            c = self.__get_ns_by_id(nid)
             ret[c.name] = c
         return ViewOnlyDict(ret)
 
@@ -313,6 +425,8 @@ class Node(ModelElement):
         :return:
         """
         # immediately-attached interfaces
+        # FIXME: note that because there could be a collision on interface name, this
+        # may produce invalid results sometimes. Better to use list_of_interfaces
         node_if_list = self.topo.graph_model.get_all_node_or_component_connection_points(parent_node_id=self.node_id)
         direct_interfaces = dict()
         for nid in node_if_list:
@@ -324,12 +438,20 @@ class Node(ModelElement):
             direct_interfaces.update(comp_interfaces)
         return ViewOnlyDict(direct_interfaces)
 
-    def __list_of_interfaces(self) -> Tuple[Any]:
+    def __list_of_interfaces(self) -> Tuple[Interface]:
         """
         List all interfaces of node and its components
         :return:
         """
-        return tuple(self.__list_interfaces().values())
+        node_if_list = self.topo.graph_model.get_all_node_or_component_connection_points(parent_node_id=self.node_id)
+        direct_interfaces = list()
+        for nid in node_if_list:
+            i = self.__get_interface_by_id(nid)
+            direct_interfaces.append(i)
+        cdict = self.__list_components()
+        for k, v in cdict.items():
+            direct_interfaces.extend(v.interface_list)
+        return tuple(direct_interfaces)
 
     def get_component(self, name: str):
         """
@@ -339,27 +461,6 @@ class Node(ModelElement):
         """
         return self.__get_component_by_name(name)
 
-    def __getattr__(self, item):
-        """
-        Special handling for attributes like 'components' and 'interfaces' -
-        which query into the model. They return dicts and list
-        containers. Modifying containers does not affect the underlying
-        graph mode, but modifying elements of lists or values of dicts does.
-        :param item:
-        :return:
-        """
-        if item == 'components':
-            return self.__list_components()
-        if item == 'interfaces':
-            return self.__list_interfaces()
-        if item == 'interface_list':
-            return self.__list_of_interfaces()
-        if item == 'direct_interfaces':
-            return self.__list_direct_interfaces()
-        if item == 'switch_fabrics':
-            return self.__list_switch_fabrics()
-        raise RuntimeError(f'Attribute {item} not available')
-
     def __repr__(self):
         _, node_properties = self.topo.graph_model.get_node_properties(node_id=self.node_id)
         node_sliver = self.topo.graph_model.node_sliver_from_graph_properties_dict(node_properties)
